/******************************
 *   
 *   Copyright (c) ASITEQ,Inc.
 *   ALL RIGHTS RESERVED
 *
******************************/
#include <common.h>
#include <regs.h>
#include "s3c2416_spi.h"

#define 		SPI_WR		0
#define 		SPI_RD		1

#define 		SDI		(1<<11)	
#define 		SCK		(1<<13)		
#define 		FRM	    	(1<<6)	
#define 		SDO		(1<<12)	

/* BOE 3" LCD setting */
static unsigned char data[78]=
{
	0x83,0X02,
	0x85,0X03,
	0x8B,0X00,
	0x8C,0X93,
	0x91,0X01,
	0x83,0X00,

	/*GAMMA SETTING*/
	0x3E,0xF0,
	0x3F,0x07,
	0x40,0X00,
	0x41,0X43,
	0x42,0X16,
	0x43,0X16,
	0x44,0X43,
	0x45,0X77,
	0x46,0X00,
	0x47,0X1E,
	0x48,0X0F,
	0x49,0X00,

	/*USER SETTING*/
	0x17,0X91,	/* OSC Control 1 Register */
	0x23,0X01,	/* Display Control 1 Register */
	/*POWER SUPPLY SETTING*/
	0x2B,0XF9,
	0x1B,0X14,
	0x1A,0X11,
	0x1C,0X0D,
	0x1F,0X27,
	0x19,0X0A,
	0x19,0X1A,
	0x19,0X12,
	0x1E,0X2C,
	0x5A,0X01,
        /*DISPLAY ON*/
	0x3C,0XC0,
	0x3D,0XC0,
	0x34,0X38,
	0x35,0X38,
	0x24,0X38,
	/* 40ms delay or more*/
        0x24,0X3C,	/* Display Control 2 Register */
#if 0
	0x16,0Xd8,	/* MY:1 MX:1 GS:1 BGR:1 */
#else
	0x16,0X08,	/* MY:0 MX:0 GS:0 BGR:1 */
#endif
	0x01,0X02,
	0x55,0X00
};

unsigned long subdelay(unsigned int a, unsigned int b)
{
	return (a + b + 10) * a;
}

unsigned long subdelay2(unsigned int a, unsigned int b)
{
	int		i;
	for (i = 0; i < 3; i++) {
		a += i;
		b = i;
		if (i/2)
			a = subdelay(a, b);
		else
			b = subdelay(b, a);
	}
	return b;
}

static unsigned long a_temp, b_temp;
void adelay(unsigned int count)
{
	int		i, j;
	for (j = 0; j < count; j++) {
		for (i = 0; i <5 ; i++) {
			a_temp = i+1;
			b_temp = i;
			if (i/2 )
				a_temp = subdelay2(a_temp, b_temp);
			else
				b_temp = subdelay2(b_temp, a_temp);
		}
	}
}


static void sdi_low(void)
{
	GPEDAT_REG &= ~SDI;	
}
static void sdi_hi(void)
{
	GPEDAT_REG |= SDI;
}
static void sck_low(void)
{
	GPEDAT_REG  &= ~SCK;
}
static void sck_hi(void)
{
	GPEDAT_REG |= SCK;
}
static void frm_low(void)
{
	GPEDAT_REG  &= ~FRM;
}
static void frm_hi(void)
{
	GPEDAT_REG |= FRM;
}
static void sdo_low(void)
{
	GPEDAT_REG  &= ~SDO;
}
static void sdo_hi(void)
{
	GPEDAT_REG |= SDO;
}

static void spi_out_byte(unsigned char *ptr)
{	
	unsigned char i, temp;

	for(i=0;i<8;i++)
	{
		temp = *ptr << i;
		temp &= 0x80;
		if(temp){
			sdo_hi();
		}else{
			sdo_low();
		}
		adelay(2);
		sck_low();
		adelay(4);
		sck_hi();
		adelay(2);
	}
}


static void send_spi(unsigned char deviceID,unsigned char data)
{
	unsigned char tmp;
	tmp = deviceID;
	tmp |= SPI_WR; 
	frm_low();
	adelay(20);//yup100= 10
	spi_out_byte(&tmp);
	spi_out_byte(&data);
	sdo_hi();
	adelay(20);
	frm_hi();
	adelay(20);
}

void s3c2416_spi_init(void)
{
	unsigned char i,tmp;
	/*LCD Reset GPIOB 1*/
	GPBDAT_REG |= (1<<1);	
	adelay(100);	
	for(i=0;i<78;i++){
		if(i%2){
			tmp=0x76;
		}
		else {
			tmp=0x74;
		}
		send_spi(tmp,data[i]);
		if(i==69)adelay(16000);//100ms
	}
}


