/********************************************
*	FUNCTION	: LCD Functions
*	MODEL		: YUP-150 
*	CPU	NAME	: S3C2416
*	SOURCE		: yup150_lcd.c
*	DEVISION	: DEPT. H/W TEAM
*			: ASITEQ, INC.  
*********************************************/

#include <common.h>
#include <s3c2416.h>
#include "display.h"
#include "s3c2416_spi.h"
//#include <WVGA_SmileAgain_bak1.h>
//#include <asiteq24.h>


//Select the window # which you want to test
//0: Window 0   1: Window 1
#define WINDOW_SEL 0
//Select the buffer # which you want to test
// 0: Buffer 0   1: Buffer 1
#define BUFFER_SEL 0

#ifdef CONFIG_ASI_LOGO
#include <asi_logo.h>
#endif

#define Delay(x) udelay(x*1000)

#define H_RESOLUTION    240 	/* horizon pixel  x resolition */
#define V_RESOLUTION    432 	/* line cnt       y resolution */

#define H_FP        6       	/* front porch */
#define H_SW        41       	/* Hsync width */
#define H_BP        3      	/* Back porch */

#define V_FP        5       	/* front porch */
#define V_SW        10       	/* Vsync width */
#define V_BP        8       	/* Back porch */

#define lcd_frame_rate 1
#define lcd_burst_mode 0 //16word burst mode,8word 1,4word 2
#define SetEPLL(Mdiv, Pdiv, Sdiv) EPLLCON_REG = ((Mdiv <<16)|(Pdiv <<8)|(Sdiv))


unsigned int logo_data_size = 0;
unsigned int *lcd_framebuffer=0;

unsigned short screen_width, screen_height;

unsigned char lcd_burst_in_byte[]=
{
	    64,32,16
};

unsigned int lcd_frame_buffer[][2]=
{
	    {LCD_WIN0_FRAME_BUFFER0,LCD_WIN0_FRAME_BUFFER1},
	    {LCD_WIN1_FRAME_BUFFER, NULL}
};

volatile unsigned int  *WINCONx_Reg_Addr[]=
{
	   (volatile unsigned int *)&WINCON0_REG,  (volatile unsigned int *)&WINCON1_REG 
}; 
volatile unsigned int *VIDOSDxA_Reg_Addr[]=
{
	    (volatile unsigned int *)&VIDOSD0A_REG, (volatile unsigned int *)&VIDOSD1A_REG
};

volatile unsigned int *VIDOSDxB_Reg_Addr[]=
{
	    (volatile unsigned int *)&VIDOSD0B_REG, (volatile unsigned int *)&VIDOSD1B_REG
};

volatile unsigned int *VIDOSDxC_Reg_Addr[]=
{
	    NULL, (volatile unsigned int *)&VIDOSD1C_REG
};


volatile unsigned int *VIDWxADD0_Reg_Addr[][2]=
{
	    {(volatile unsigned int *)&VIDW00ADD0B0_REG, (volatile unsigned int *)&VIDW00ADD0B1_REG},
	    {(volatile unsigned int *)&VIDW01ADD0_REG, NULL}
};

volatile unsigned int *VIDWxADD1_Reg_Addr[][2]=
{
	    {(volatile unsigned int *)&VIDW00ADD1B0_REG, (volatile unsigned int *)&VIDW00ADD1B1_REG},
	    {(volatile unsigned int *)&VIDW01ADD1_REG, NULL}
};

volatile unsigned int *VIDWxADD2_Reg_Addr[][2]=
{
	    {(volatile unsigned int *)&VIDW00ADD2B0_REG, (volatile unsigned int *)&VIDW00ADD2B1_REG},
	    {(volatile unsigned int *)&VIDW01ADD2_REG, NULL}
};

volatile unsigned int *WxKEYCON0_Reg_Addr[]=
{
	    0, (volatile unsigned int *)&W1KEYCON0_REG
};

volatile unsigned int *WxKEYCON1_Reg_Addr[]=
{
	    0, (volatile unsigned int *)&W1KEYCON1_REG
};

volatile unsigned int *WINxMAP_Reg_Addr[]=
{
	    (volatile unsigned int *)&WIN0MAP_REG, (volatile unsigned int *)&WIN1MAP_REG
};




void (*PutPixel)(unsigned int x,unsigned int y,unsigned int c);


void lcd_blight_set(unsigned int value)
{
	unsigned int i;
	for(i=0; i<value;i++){
		GPBDAT_REG &= ~(1<<0);
		udelay(300);
		GPBDAT_REG |= (1<<0);
		udelay(150);
	}
	if(value>3)printf("LCD BACKLIGHT ON\n");
}

void lcd_blight_off(void)
{
	GPBDAT_REG &= ~(1<<0);
}


void set_lcd_port(void)
{
 	GPCCON_REG |= 0xaaaa02a9;  	// CTRL,  VD[7:0]
    	GPDCON_REG |= 0xaaaaaaaa;  	// VD[23:8]
	GPCPU_REG = 0;   		// CTRL,  VD[7:0]
	GPDPU_REG = 0;   		// CTRL,  VD[7:0]
}

char* Select_Win_Buf(void)
{
	static char winbuf[2];
	winbuf[0] = WINDOW_SEL;
	if (winbuf[0]==0)
		winbuf[1] = BUFFER_SEL;
	else
		winbuf[1]=0;
	return &winbuf[0];
}


void LcdEnvidOnOff(unsigned char onoff)
{
	if(onoff==LCD_ON)
		VIDCON0_REG |= (3);        // ENVID On using Per Frame method
	else
	{
		VIDCON0_REG &= (~1);       // ENVID Off using Per Frame method
		while(VIDCON0_REG & 1);   // Check end of frame
	}
}

void LcdWindowOnOff(unsigned char num, unsigned char onoff)
{
	switch(num)
	{
		case LCD_WIN_0:
			if(onoff==LCD_ON)
				WINCON0_REG |= 0x01;
			else
				WINCON0_REG &= ~0x01;
			break;
		case LCD_WIN_1:
			if(onoff==LCD_ON)
				WINCON1_REG |= 0x01;
			else
				WINCON1_REG &= ~0x01;
			break;
		case LCD_WIN_ALL:
			if(onoff==LCD_ON)
			{
				WINCON0_REG |= 0x01;
				WINCON1_REG |= 0x01;
			}
			else
			{
				WINCON0_REG &= ~0x01;
				WINCON1_REG &= ~0x01;
			}
		default:
			break;
	}
	return;
}


void Display_Start(char win_num)// 0: LCD_WIN_0, 1: LCD_WIN_1, 2: LCD_WIN_ALL
{
	LcdWindowOnOff(win_num,LCD_ON);
	LcdEnvidOnOff(LCD_ON);
	return;
}

void Display_End(char win_num)
{	
//	lcd_blight_off();//lcd backlight off
	LcdWindowOnOff(win_num,LCD_OFF);
	LcdEnvidOnOff(LCD_OFF);
	return;
}



//===================================================
// 
// General Library for Font and drawing...
//
//===================================================
inline void _PutPixel32Bit(unsigned int x,unsigned int y,unsigned int c)
{
	    if(x<screen_width && y<screen_height)
			*(unsigned int *)(lcd_framebuffer+y*screen_width+x)=c;
}
inline void _PutPixel18Bit(unsigned int x,unsigned int y,unsigned int c)
{
	    if(x<screen_width && y<screen_height)
			*(unsigned int *)(lcd_framebuffer+y*screen_width+x)=c;
}

void LCD_Rectangle(int x1,int y1,int x2,int y2,int color)
{
	LCD_Line(x1,y1,x2,y1,color);
	LCD_Line(x2,y1,x2,y2,color);
	LCD_Line(x1,y2,x2,y2,color);
	LCD_Line(x1,y1,x1,y2,color);
}

void LCD_FilledRectangle(int x1,int y1,int x2,int y2,int color)
{
	int i;
	
	for(i=y1;i<=y2;i++)
		LCD_Line(x1,i,x2,i,color);
}



// LCD display is flipped vertically
// // But, think the algorithm by mathematics point.
// //   3I2
// //   4 I 1
// //  --+--   <-8 octants  mathematical cordinate
// //   5 I 8
// //   6I7



void LCD_Line(int x1,int y1,int x2,int y2,int color)
{
	int dx,dy,e;
	dx=x2-x1;
	dy=y2-y1;

	if(dx>=0)
	{
		if(dy >= 0) // dy>=0
		{
			if(dx>=dy) // 1/8 octant
			{
				e=dy-dx/2;
				while(x1<=x2)
				{
					PutPixel(x1,y1,color);
					if(e>0){y1+=1;e-=dx;}
					x1+=1;
					e+=dy;
				}
			}
			else        // 2/8 octant
			{
				e=dx-dy/2;
				while(y1<=y2)
				{
					PutPixel(x1,y1,color);
					if(e>0){x1+=1;e-=dy;}y1+=1;
					e+=dx;
				}
			}
		}
		else           // dy<0
		{
			dy=-dy;   // dy=abs(dy)
			if(dx>=dy) // 8/8 octant
			{
				e=dy-dx/2;
				while(x1<=x2)
				{
					PutPixel(x1,y1,color);
					if(e>0){y1-=1;e-=dx;}
					x1+=1;
					e+=dy;
				}
			}
			else        // 7/8 octant
			{
				e=dx-dy/2;
				while(y1>=y2)
				{
					PutPixel(x1,y1,color);
					if(e>0){x1+=1;e-=dy;}
					y1-=1;
					e+=dx;
				}
			}
		}
	}
	else //dx<0
	{
		dx=-dx;     //dx=abs(dx)
		if(dy >= 0) // dy>=0
		{
			if(dx>=dy) // 4/8 octant
			{
				e=dy-dx/2;
				while(x1>=x2)
				{
					PutPixel(x1,y1,color);
					if(e>0){y1+=1;e-=dx;}
					x1-=1;
					e+=dy;
				}
			}
			else        // 3/8 octant
			{
				e=dx-dy/2;
				while(y1<=y2)
				{
					PutPixel(x1,y1,color);
					if(e>0){x1-=1;e-=dy;}
					y1+=1;
					e+=dx;
				}
			}
		}
		else           // dy<0
		{
			dy=-dy;   // dy=abs(dy)
			if(dx>=dy) // 5/8 octant
			{
				e=dy-dx/2;
				while(x1>=x2)
				{
					PutPixel(x1,y1,color);
					if(e>0){y1-=1;e-=dx;}
					x1-=1;
					e+=dy;
				}
			}
			else        // 6/8 octant
			{
				e=dx-dy/2;
				while(y1>=y2)
				{
					PutPixel(x1,y1,color);
					if(e>0){x1-=1;e-=dy;}
					y1-=1;
					e+=dx;
				}
			}
		}
	}
}


		
// Frame buffer address setting : Windows0 have the two buffer like as buffer0, buffer1. 
// Each buffer have the register for setting the start and end address. But Window1 have one buffer
// base lcd_framebuffer = Noncache address(0x31000000)
void Basic_Display_Setting( int win_num,int buf_num, int bpp, unsigned int width, unsigned int height)
{
	unsigned int screenwidth_in_byte=0,offsize_in_byte=0;
	int pagewidth,offsize,pageheight;
	
	//PutPixel=_PutPixel32Bit;// 24bit color 888
	PutPixel=_PutPixel18Bit;// 18bit color 666 
	
	screen_width=width;     // Frame Buffer hoz. size
	screen_height=height;   // Frame Buffer ver. size
	
	pagewidth= H_RESOLUTION;// Real LCD hoz. size
	pageheight=V_RESOLUTION;  // Real LCD ver. size
	
	offsize = screen_width - pagewidth;// 0=800-800
	if(offsize<0)// LCD size > frame buffer
	{
		offsize=0;
		pagewidth=screen_width;
		pageheight=screen_height;
	}
	screenwidth_in_byte = screen_width*32/8;//32=>lcd_bit_order 3200= 800*32/8
	if((screenwidth_in_byte%lcd_burst_in_byte[lcd_burst_mode])!=0)
	{
		screenwidth_in_byte += lcd_burst_in_byte[lcd_burst_mode] - (screenwidth_in_byte%lcd_burst_in_byte[lcd_burst_mode]);
	}
	offsize_in_byte = offsize*32/8;//32=>lcd_bit_order 0=0*32/8
	// WINCON0/1
    	*WINCONx_Reg_Addr[win_num] = \
				     (buf_num<<WINCON_BUFSEL)|(0<<WINCON_BUFAUTOEN)|(0<<WINCON_SWAP_S)|(lcd_burst_mode<<WINCON_BURSTLEN_S)|(bpp<<WINCON_BPP_S);
    	// WIN0/1 OSD(top)
    	*VIDOSDxA_Reg_Addr[win_num] = (0<<VIDOSDxAB_HORIZON_X_S)|(0);
    	// WIN0/1 OSD(bottom)
    	*VIDOSDxB_Reg_Addr[win_num] = ((pagewidth-1)<<VIDOSDxAB_HORIZON_X_S)|(pageheight-1);
	lcd_framebuffer = (unsigned int *)lcd_frame_buffer[win_num][buf_num];


    	// WIN0/1 buffer start address
	*VIDWxADD0_Reg_Addr[win_num][buf_num] = (unsigned int)lcd_framebuffer;
	// WIN0/1 buffer end address
	*VIDWxADD1_Reg_Addr[win_num][buf_num] = (unsigned int)lcd_framebuffer + (screenwidth_in_byte+offsize_in_byte)*(pageheight);
	// WIN0/1 buffer size 
	*VIDWxADD2_Reg_Addr[win_num][buf_num] = (offsize_in_byte<<VIDWxADD2_OFFSET_SIZE_S)|screenwidth_in_byte;
	
    	*WINxMAP_Reg_Addr[win_num] = 0;
	if (win_num>0)
	{
		*VIDOSDxC_Reg_Addr[win_num] = 0;
		*WxKEYCON0_Reg_Addr[win_num] = 0;
		*WxKEYCON1_Reg_Addr[win_num] = 0;
	}
}



void Make_Image(char window, char buffer, char image)   // 0: 24bpp Color bar 1: Picturs for each size and type
{
#if 0
	unsigned int i,j,k=0;
	if(image==1)
	{
		Basic_Display_Setting(window,buffer,WINCONx_18BPP_666,H_RESOLUTION,V_RESOLUTION);
		logo_data_size = sizeof(pSmileAgain);
		for(j=0;j<screen_height;j++)for(i=0;i<screen_width;i++)PutPixel(i,j,pSmileAgain[k++]);
	}
	else if(image ==2)   //RGB red
	{
		Basic_Display_Setting(window,buffer,WINCONx_18BPP_666,H_RESOLUTION,V_RESOLUTION);
			
		for(j=0;j<screen_height;j++)for(i=0;i<screen_width;i++)PutPixel(i,j,0xff0000);
	}
	else if(image ==3)   //RGB green 
		{
			Basic_Display_Setting(window,buffer,WINCONx_18BPP_666,H_RESOLUTION,V_RESOLUTION);
			
			for(j=0;j<screen_height;j++)for(i=0;i<screen_width;i++)PutPixel(i,j,0x00ff00);
	}
	else if(image ==4)   //RGB blue 
		{
			Basic_Display_Setting(window,buffer,WINCONx_18BPP_666,H_RESOLUTION,V_RESOLUTION);
			
			for(j=0;j<screen_height;j++)for(i=0;i<screen_width;i++)PutPixel(i,j,0x0000ff);
	}
	else  //RGB bar(18bpp)
	{
#endif
/* color bar */
#if 0
		Basic_Display_Setting(window,buffer,WINCONx_18BPP_666,H_RESOLUTION,V_RESOLUTION);
		LCD_FilledRectangle(0, 0, screen_width-1, screen_height/4*1-1, 0x3f000);
		LCD_FilledRectangle(0, screen_height/4*1, screen_width-1, screen_height/4*2-1, 0x00fc0);
		LCD_FilledRectangle(0, screen_height/4*2, screen_width-1, screen_height/4*3-1, 0x0003f);
		LCD_FilledRectangle(0, screen_height/4*3, screen_width-1, screen_height-1, 0x3ffff);
//		lprintf(screen_width/2,screen_height/2,0xffffff,"Hello~");
#endif

/* red color */
		Basic_Display_Setting(window,buffer,WINCONx_18BPP_666,H_RESOLUTION,V_RESOLUTION);
		LCD_FilledRectangle(0, 0, screen_width-1, screen_height-1, 0x3f000);
		
#if 0
	}
#endif

#if 0
	INTMSK_REG &= ~(1<<16);
	VIDINTCON_REG = (0x3f<<20)|(1<<15)|(1<<12)|(1<<0); //Frame(VSYNC)
	INTSUBMSK_REG &= ~(1<<16);//RGB I/F Frame int.
#endif
}

#if 0
void yup_lcd_init(void)
{
	unsigned char clkval=0;
	unsigned short h_cnt,v_cnt;
	unsigned int vclk;
	
	char image,window,buffer;
	char* winbuf;

	set_lcd_port();
	s3c2416_spi_init();	
	
	/************* Parallel interface *************/	
	VIDCON0_REG = VIDCON0_S_RGB_IF | \
				  VIDCON0_S_RGB_PAR| \
				  VIDCON0_S_VCLK_GATING_OFF|\
				  VIDCON0_S_CLKDIR_DIVIDED|\
				  VIDCON0_S_CLKSEL_HCLK;
    	v_cnt = (V_BP + V_FP + V_SW + V_RESOLUTION);
    	h_cnt = (H_BP + H_FP + H_SW + H_RESOLUTION);

	/**********************************************/
	//HCLK = get_HCLK();
	//printf("HCLK = %08lu\n", HCLK);
	/************* Vclk setting ******************/	
	if(VIDCON0_REG & (1<<2))   //EPLL
	{
		CLKDIV1CON_REG &= ~(0xf<<8);
	   	CLKDIV1CON_REG |= (0x2<<8);   // 400/3(133MHz)
		SetEPLL( 400, 3, 2);    // EPLL 400MHz
		EPLLCON_REG &=~(1<<24);     // EPLL ON 
		CLKSRCCON_REG |=(1<<6);        // EPLL Output
		clkval = (unsigned char)(((float)get_HCLK()/(float)(h_cnt*v_cnt*lcd_frame_rate))+0.5)-1;

	} else {   //HCLK
		clkval = (unsigned char)( ((float)get_HCLK()/(float)(v_cnt*h_cnt*lcd_frame_rate) )+0.5)-1;
	}

	clkval =0xe;//c=>10256khz d=>9523khz e=8888khz
	vclk = (get_HCLK()/(clkval+1))/1000;
	printf("VCLK: %dKHz\n",vclk);
	printf("CLKVAL: %x\n",clkval);

	if(clkval > 0x3f) printf("Check frame rate: over flow !\n");
	VIDCON0_REG |= (clkval <<VIDCON0_CLKVAL_F_SHIFT);
	VIDCON1_REG = VIDCON1_S_HSYNC_INVERTED|VIDCON1_S_VSYNC_INVERTED;
VIDTCON0_REG =((V_BP-1)<<VIDTCON0_BPD_S)|((V_FP-1)<<VIDTCON0_FPD_S)|(V_SW-1);
	VIDTCON1_REG =((H_BP-1)<<VIDTCON0_BPD_S)|((H_FP-1)<<VIDTCON0_FPD_S)|(H_SW-1);
	VIDTCON2_REG = ((V_RESOLUTION-1)<<VIDTCON2_LINEVAL_S)|(H_RESOLUTION-1);

	/*******************************************/
	winbuf=Select_Win_Buf();
	window=*winbuf;
	buffer=*(winbuf+1);
	//select the image which you want to display
	// 0: RGB bar display(18bpp)
	// 1: 24bpp Landscape(Valley in Jeju island)
	image = 0; 
	Make_Image(window, buffer, image);
	Display_Start(window);

//		Display_End(window);
#if 0
	VIDINTCON_REG = (0x3f<<20)|0;
	INTMSK_REG |= (1<<16);
	INTSUBMSK_REG |= (1<<16);//RGB I/F Frame int.
#endif
#if 0	
	Display_End(window);
	VIDINTCON_REG = (0x3f<<20)|0;
	INTMSK_REG |= (1<<16);
#endif
}
#else
void yup_lcd_init(void)
{
	unsigned char clkval=0;
	unsigned short h_cnt,v_cnt;
	unsigned int vclk;
	
	char image,window,buffer;
	char* winbuf;

	set_lcd_port();


	
	/************* Parallel interface *************/	
	VIDCON0_REG = VIDCON0_S_RGB_IF | \
				  VIDCON0_S_RGB_PAR| \
				  VIDCON0_S_VCLK_GATING_OFF|\
				  VIDCON0_S_CLKDIR_DIVIDED|\
				  VIDCON0_S_CLKSEL_HCLK;
    	v_cnt = (V_BP + V_FP + V_SW + V_RESOLUTION);
    	h_cnt = (H_BP + H_FP + H_SW + H_RESOLUTION);

	/**********************************************/
	//HCLK = get_HCLK();
	//printf("HCLK = %08lu\n", HCLK);
	/************* Vclk setting ******************/	
	if(VIDCON0_REG & (1<<2))   //EPLL
	{
		CLKDIV1CON_REG &= ~(0xf<<8);
	   	CLKDIV1CON_REG |= (0x2<<8);   // 400/3(133MHz)
		SetEPLL( 400, 3, 2);    // EPLL 400MHz
		EPLLCON_REG &=~(1<<24);     // EPLL ON 
		CLKSRCCON_REG |=(1<<6);        // EPLL Output
		clkval = (unsigned char)(((float)get_HCLK()/(float)(h_cnt*v_cnt*lcd_frame_rate))+0.5)-1;

	} else {   //HCLK
		clkval = (unsigned char)( ((float)get_HCLK()/(float)(v_cnt*h_cnt*lcd_frame_rate) )+0.5)-1;
	}

	clkval =0xe;//c=>10256khz d=>9523khz e=8888khz
	vclk = (get_HCLK()/(clkval+1))/1000;
	printf("VCLK: %dKHz\n",vclk);
	printf("CLKVAL: %x\n",clkval);

	if(clkval > 0x3f) printf("Check frame rate: over flow !\n");
	VIDCON0_REG |= (clkval <<VIDCON0_CLKVAL_F_SHIFT);
	VIDCON1_REG = VIDCON1_S_HSYNC_INVERTED|VIDCON1_S_VSYNC_INVERTED;
	VIDTCON0_REG =((V_BP-1)<<VIDTCON0_BPD_S)|((V_FP-1)<<VIDTCON0_FPD_S)|(V_SW-1);
	VIDTCON1_REG =((H_BP-1)<<VIDTCON0_BPD_S)|((H_FP-1)<<VIDTCON0_FPD_S)|(H_SW-1);
	VIDTCON2_REG = ((V_RESOLUTION-1)<<VIDTCON2_LINEVAL_S)|(H_RESOLUTION-1);

	/*******************************************/
	winbuf=Select_Win_Buf();
	window=*winbuf;
	buffer=*(winbuf+1);
	//select the image which you want to display
	// 0: RGB bar display(18bpp)
	// 1: 24bpp Landscape(Valley in Jeju island)
	image = 0; 
	Make_Image(window, buffer, image);
	Display_Start(window);

//		Display_End(window);
#if 0
	VIDINTCON_REG = (0x3f<<20)|0;
	INTMSK_REG |= (1<<16);
	INTSUBMSK_REG |= (1<<16);//RGB I/F Frame int.
#endif
#if 0	
	Display_End(window);
	VIDINTCON_REG = (0x3f<<20)|0;
	INTMSK_REG |= (1<<16);
#endif


}
#endif
