#include <common.h>

#ifdef CONFIG_YUP150

#include <command.h>
#include <nand.h>
#ifdef CONFIG_MMC
#include <mmc.h>
#endif /* CONFIG_MMC */


// (3~64)bright settign max64 but 53(max 24.x mA) used
#define lcd_bright_val 52  


extern int do_bootm(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[]);

struct sys_flash_img_info {
    unsigned char	magic[4];	// 0	(0x32 0x18 0x20 0x82)
    unsigned char	reserved1[4];	// 4
    unsigned char	date[4];	// 8	(년,월,일,시간)

    unsigned char	reserved2[40];	// 12

    unsigned int        skernel_lba;	// 52
    unsigned int        skernel_nblk;	// 56
    unsigned int        skernel_addr;	// 60
    unsigned int	skernel_crc;	// 64

    unsigned int	sroot_lba;	// 68
    unsigned int	sroot_nblk;	// 72
    unsigned int	sroot_addr;	// 76
    unsigned int	sroot_crc;	// 80
    unsigned int	sramdisk_size;	// 84

    unsigned char	ssysimg_rev[4];	// 88

    unsigned int	random1[104];	// 92

    unsigned int	info_crc;	// 508
};
#define IMGINFOCRCINIT	0x84692565

#define IMG_ALIGN_SIZE 2048

#ifdef CONFIG_ENABLE_MMU
#define KERNEL_ADDR	 0xc0008000
#define KERNEL_ADDR_STR	"0xc0008000"
#define INITRD_ADDR	 0xc0800000
#define INITRD_ADDR_STR	"0xc0800000"
#else /* CONFIG_ENABLE_MMU */
#define KERNEL_ADDR	 0x30008000
#define KERNEL_ADDR_STR	"0x30008000"
#define INITRD_ADDR	 0x30800000
#define INITRD_ADDR_STR	"0x30800000"
#endif /* CONFIG_ENABLE_MMU */

static int read_data_from_nand(nand_info_t *nand, ulong addr, ulong *off,
			       ulong size)
{
	ulong	nr, onr;
	while (size) {
		while (nand_block_isbad(nand, *off)) {
			printf("Skip bad block : %08x\n", *off);
			*off += nand->erasesize;
		}
		nr = size;
		if (nr > (nand->erasesize - (*off % nand->erasesize)))
			nr = nand->erasesize - (*off % nand->erasesize);
		onr = nr;
		if (nand_read(nand, *off, &nr, (u_char *) addr))
			return -1;

		addr += onr;
		*off += onr;
		size -= onr;
	}

	return 0;
}

static int load_sysimg_from_nand(nand_info_t *nand, ulong off)
{
	ulong crc;
	ulong lba;
	struct sys_flash_img_info	*info =
		(struct sys_flash_img_info *) (KERNEL_ADDR - IMG_ALIGN_SIZE);

	/*
	 * read header
	 */
	if (read_data_from_nand(nand, (ulong) info, &off, IMG_ALIGN_SIZE))
		return -1;
	if (info->magic[0] != 0x32 || info->magic[1] != 0x18 ||
	    info->magic[2] != 0x20 || info->magic[3] != 0x82)
	{
		printf("Info Magic Error : %02X %02X %02X %02X\n",
			info->magic[0], info->magic[1],
			info->magic[2], info->magic[3]);
		return -1;
	}
	crc = crc32(IMGINFOCRCINIT, (unsigned char *) info, 512 - 4);
	if (crc != info->info_crc) {
		printf("Info CRC Error(calced, stored) : %08X, %08X\n",
			crc, info->info_crc);
		return -1;
	}

	/* read kernel */
	if (info->skernel_lba != IMG_ALIGN_SIZE / 512) {
		printf("Info Kernel LBA is incorrect : %08X\n",
			info->skernel_lba);
		return -1;
	}
	if (read_data_from_nand(nand, KERNEL_ADDR, &off,
				info->skernel_nblk * 512))
	{
		printf("Kernel Image read error\n");
		return -1;
	}
	crc = crc32(0, (unsigned char *) KERNEL_ADDR,
		    info->skernel_nblk * 512);
	if (crc != info->skernel_crc) {
		printf("Kernel CRC Error(calced, stored) : %08X, %08X\n",
			crc, info->skernel_crc);
		return -1;
	}
	if (off % IMG_ALIGN_SIZE)
		off += IMG_ALIGN_SIZE - (off % IMG_ALIGN_SIZE);

	/* read ramdisk */
	lba = IMG_ALIGN_SIZE / 512 + info->skernel_nblk;
	if (lba % (IMG_ALIGN_SIZE / 512))
		lba += (IMG_ALIGN_SIZE / 512) - (lba % (IMG_ALIGN_SIZE / 512));
	if (info->sroot_lba != lba) {
		printf("Info Initrd LBA is incorrect : %08X\n",
			info->sroot_lba);
		return -1;
	}
	if (read_data_from_nand(nand, INITRD_ADDR, &off,
				info->sroot_nblk * 512))
	{
		printf("Initrd Image read error\n");
		return -1;
	}
	crc = crc32(0, (unsigned char *) INITRD_ADDR,
		    info->sroot_nblk * 512);
	if (crc != info->sroot_crc) {
		printf("Initrd CRC Error(calced, stored) : %08X, %08X\n",
			crc, info->sroot_crc);
		return -1;
	}

	return 0;
}

#ifdef CONFIG_MMC
#include <fat.h>
#include <movi.h>
extern block_dev_desc_t *  mmc_get_dev(int dev);
static int load_sysimg_from_mmc(void)
{
	block_dev_desc_t *dev_desc = NULL;
	long size;
	ulong crc;
	ulong lba;
	struct sys_flash_img_info	*info =
		(struct sys_flash_img_info *) (KERNEL_ADDR - IMG_ALIGN_SIZE);

//	dev_desc = get_dev("mmc", 1);
	dev_desc = mmc_get_dev(0);
	if (dev_desc == NULL) {
		puts ("\n** Invalid boot device **\n");
		return -1;
	}
	if (fat_register_device(dev_desc, 1) != 0) {
		puts ("\n** Unable to use mmc 1:1 for fatload **\n");
		return -1;
	}
	size = file_fat_read("/boot/yup150/system.img",
			     (unsigned char *) info, 0);
	if (size < 0) {
		puts("\n** Unable to read /boot/yup150/system.img from mmc 1:1\n");
		return -1;
	}
	printf ("\n%ld bytes read\n", size);

	if (info->magic[0] != 0x32 || info->magic[1] != 0x18 ||
	    info->magic[2] != 0x20 || info->magic[3] != 0x82)
	{
		printf("Info Magic Error : %02X %02X %02X %02X\n",
			info->magic[0], info->magic[1],
			info->magic[2], info->magic[3]);
		return -1;
	}
	crc = crc32(IMGINFOCRCINIT, (unsigned char *) info, 512 - 4);
	if (crc != info->info_crc) {
		printf("Info CRC Error(calced, stored) : %08X, %08X\n",
			crc, info->info_crc);
		return -1;
	}
	printf("Info CRC Matched(calced, stored) : %08X, %08X\n",
		crc, info->info_crc);

	/* ramdisk move */
	memmove(INITRD_ADDR, (char *) info + info->sroot_lba * 512,
		info->sroot_nblk * 512);
	crc = crc32(0, (unsigned char *) INITRD_ADDR,
		    info->sroot_nblk * 512);
	if (crc != info->sroot_crc) {
		printf("Initrd CRC Error(calced, stored) : %08X, %08X\n",
			crc, info->sroot_crc);
		return -1;
	}
	printf("Initrd CRC Matched(calced, stored) : %08X, %08X\n",
		crc, info->sroot_crc);

	/* kernel move */
	memmove(KERNEL_ADDR, (char *) info + info->skernel_lba * 512,
		info->skernel_nblk * 512);

	crc = crc32(0, (unsigned char *) KERNEL_ADDR,
		    info->skernel_nblk * 512);
	if (crc != info->skernel_crc) {
		printf("Kernel CRC Error(calced, stored) : %08X, %08X\n",
			crc, info->skernel_crc);
		return -1;
	}
	printf("Kernel CRC Matched(calced, stored) : %08X, %08X\n",
		crc, info->skernel_crc);

	return 0;
}
#endif /* CONFIG_MMC */

extern nand_info_t nand_info[];       /* info for NAND chips */
static int do_try_boot_from_nand(cmd_tbl_t *cmdtp)
{
	char	*bootm_argv[] = {"bootm", KERNEL_ADDR_STR};

	printf("Trying to load system image from front partition\n");
	if (load_sysimg_from_nand(nand_info, 0x00200000) == 0)
		do_bootm(cmdtp, 0, 2, bootm_argv);

	printf("Trying to load system image from rear partition\n");
	if (load_sysimg_from_nand(nand_info, 0x01A00000) == 0)
		do_bootm(cmdtp, 0, 2, bootm_argv);

	return -1;
}

extern void yup_lcd_init(void);

int do_yup150_boot(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	char	*bootm_argv[] = {"bootm", KERNEL_ADDR_STR};
	
	s3c2416_spi_init();
	lcd_blight_set(lcd_bright_val);
	/*
	 * try boot from NAND
	 */
	do_try_boot_from_nand(cmdtp);

#ifdef CONFIG_MMC
	yup_lcd_init();

	/*
	 * try boot from mmc
	 */
	hsmmc_set_gpio();
	do {
		udelay(1000);
		hsmmc_reset();
		udelay(1000);

		movi_hc = !movi_hc;
		if (hsmmc_init() != 0) {
			printf ("No MMC card found\n");
			continue;
		}
		if (mmc_get_dev(0)->lba < 0x10000) {
			printf ("too small mmc size\n");
			continue;
		}
		if (load_sysimg_from_mmc() != 0) {
			printf ("No System image found on mmc\n");
			continue;
		}
		do_bootm(cmdtp, 0, 2, bootm_argv);
	} while (1) ;
#endif /* CONFIG_MMC */

	return 1;
}

U_BOOT_CMD(
        yup150boot,        1,      0,      do_yup150_boot,
        "yup150boot - boot from YUP100 nand/mmc card\n",
        NULL
);

#endif
