/*
 * $Id: nand_cp.c,v 1.2 2008/04/14 01:32:57 boyko Exp $
 *
 * (C) Copyright 2006 Samsung Electronics
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * You must make sure that all functions in this file are designed
 * to load only U-Boot image.
 *
 * So, DO NOT USE in common read.
 *
 * By scsuh.
 */


#include <common.h>

#if CONFIG_S3C24XX
#include <asm/io.h>
#include <linux/mtd/nand.h>
#include <regs.h>

#ifdef CONFIG_NAND_BL1_8BIT_ECC
#define USE_BL1_8BIT_ECC
#endif

#ifdef USE_BL1_8BIT_ECC 
static void nandll_print_ecc_errtype(u_char err_type)
{
	UTXH0_REG = '0' + err_type;
}

static void nandll_enable_hwecc_8bit(void)
{
	u_long nfcont, nfconf;

	/* 8 bit selection */
	nfconf = NFCONF_REG;
	nfconf &= ~(0x3 << 23);
	nfconf |= (0x1 << 23);
	NFCONF_REG = nfconf;

	/* Initialize & unlock */
	nfcont = NFCONT_REG;
	nfcont |= NFCONT_INITECC;
	nfcont &= ~NFCONT_MECCLOCK;
	nfcont &= ~NFCONT_ECC_ENC;
	NFCONT_REG = nfcont;
}

static void nandll_set_read_cmd(ulong row, ulong column, int large_block)
{
	
	NFCMD_REG = NAND_CMD_READ0;

        /* Write Address */
        NFADDR_REG = column & 0xff;

	if (large_block)
	        NFADDR_REG = (column >> 8) & 0xff;

        NFADDR_REG = (row) & 0xff;
        NFADDR_REG = (row >> 8) & 0xff;
        NFADDR_REG = (row >> 16) & 0xff;

	if (large_block)
		NFCMD_REG = NAND_CMD_READSTART;
        NF_TRANSRnB();
}

static void nandll_read_ecc_oob(uchar *buf, ulong row, ulong column, ulong size, int large_block)
{
	int i;
	nandll_set_read_cmd(row, column, large_block);
	for(i=0; i < size; i++)
               buf[i] = NFDATA8_REG;
}

static int nandll_correct_ecc(uchar *dat)
{
	int ret = -1;

	u_long nf8eccerr0, nf8eccerr1, nf8eccerr2, nfmlc8bitpt0, nfmlc8bitpt1;
	u_char err_type;

	while (NF8ECCERR0_REG & NFESTAT0_ECCBUSY) {}

	nf8eccerr0 = NF8ECCERR0_REG;
	nf8eccerr1 = NF8ECCERR1_REG;
	nf8eccerr2 = NF8ECCERR2_REG;
	nfmlc8bitpt0 = NFMLC8BITPT0_REG;
	nfmlc8bitpt1 = NFMLC8BITPT1_REG;

	err_type = (nf8eccerr0 >> 25) & 0xf;

	if ((nf8eccerr0 >> 29) & 0x1)
		err_type = 0;

	switch (err_type) {
	case 9:
		ret = -1;
		nandll_print_ecc_errtype(err_type);
		break;
	case 8:
		dat[(nf8eccerr2 >> 22) & 0x3ff] ^= ((nfmlc8bitpt1 >> 24) & 0xff);
	case 7:
		dat[(nf8eccerr2 >> 11) & 0x3ff] ^= ((nfmlc8bitpt1 >> 16) & 0xff);
	case 6:
		dat[nf8eccerr2 & 0x3ff] ^= ((nfmlc8bitpt1 >> 8) & 0xff);
	case 5:
		dat[(nf8eccerr1 >> 22) & 0x3ff] ^= (nfmlc8bitpt1 & 0xff);
	case 4:
		dat[(nf8eccerr1 >> 11) & 0x3ff] ^= ((nfmlc8bitpt0 >> 24) & 0xff);
	case 3:
		dat[nf8eccerr1 & 0x3ff] ^= ((nfmlc8bitpt0 >> 16) & 0xff);
	case 2:
		dat[(nf8eccerr0 >> 15) & 0x3ff] ^= ((nfmlc8bitpt0 >> 8) & 0xff);
	case 1:
		dat[nf8eccerr0 & 0x3ff] ^= (nfmlc8bitpt0 & 0xff);
		ret = err_type;
		nandll_print_ecc_errtype(err_type);
		break;
	case 0:
		ret = 0;
		break;
	}

	return ret;
}

static int nandll_read_subpage(uchar *buf, ulong row, ulong column, int large_block, const uchar *ecc_oob)
{
	int i;
	u_long nfcont;

	nandll_set_read_cmd(row, column, large_block);
	nandll_enable_hwecc_8bit();
	for(i=0; i < 512; i++)
                buf[i] = NFDATA8_REG;
	for(i=0; i < 13; i++)
		NFDATA8_REG = ecc_oob[i];

	nfcont = NFCONT_REG;
	nfcont |= NFCONT_MECCLOCK;
	NFCONT_REG = nfcont;
	while (!(NFSTAT_REG & NFSTAT_ECCDECDONE)) {}

	if(nandll_correct_ecc(buf)!=0)return -1;
	return 0;
}

#endif

/*
 * address format
 *              17 16         9 8            0
 * --------------------------------------------
 * | block(12bit) | page(5bit) | offset(9bit) |
 * --------------------------------------------
 */

static int nandll_read_page (uchar *buf, ulong row, int large_block)
{//buf ->1000, add->0~3c, lage->2
        int i;
	int page_size = 512;
#ifdef USE_BL1_8BIT_ECC 
	uchar ecc_oob[104];
#endif

	if (large_block == 2)
		page_size = 4096;
	else if (large_block)
		page_size = 2048;

        NAND_ENABLE_CE();

#ifdef USE_BL1_8BIT_ECC 
	nandll_read_ecc_oob(ecc_oob,row, page_size, 104, large_block);
	for (i = 0; i < (page_size/512); i++, buf += 512)
		nandll_read_subpage(buf, row, i*512, large_block, ecc_oob + 13 * i);
#else
        NFCMD_REG = NAND_CMD_READ0;

        /* Write Address */
        NFADDR_REG = 0;

	if (large_block)
	        NFADDR_REG = 0;

        NFADDR_REG = (row) & 0xff;
        NFADDR_REG = (row >> 8) & 0xff;
        NFADDR_REG = (row >> 16) & 0xff;

	if (large_block)
		NFCMD_REG = NAND_CMD_READSTART;

        NF_TRANSRnB();

	/* for compatibility(2460). u32 cannot be used. by scsuh */
	for(i=0; i < page_size; i++) {
                *buf++ = NFDATA8_REG;
        }
#endif

        NAND_DISABLE_CE();
        return 0;
}

/*
 * Read data from NAND.
 */
static int nandll_read_blocks (ulong dst_addr, ulong size, int large_block)
{
        uchar *buf = (uchar *)dst_addr;
        int i;
	uint page_shift = 9;

	if (large_block == 2)
		page_shift = 12;
	else if (large_block)
		page_shift = 11;

        /* Read pages */
        for (i = 0; i < (0x3c000>>page_shift); i++, buf+=(1<<page_shift)) 
               nandll_read_page(buf, i, large_block);
        return 0;
}

int copy_uboot_to_ram (void)
{
	int large_block = 0;
	int i;
	vu_char id;

        NAND_ENABLE_CE();
        NFCMD_REG = NAND_CMD_READID;
        NFADDR_REG =  0x00;

	/* wait for a while */
        for (i=0; i<200; i++);
	id = NFDATA8_REG;
	id = NFDATA8_REG;

	if (id == 0xD5) {
		large_block = 2;
	} else if (id > 0x80)
		large_block = 1;

	/* read NAND Block.
	 * 128KB ->240KB because of U-Boot size increase. by scsuh
	 * So, read 0x3c000 bytes not 0x20000(128KB).
	 */
	return nandll_read_blocks(CFG_PHY_UBOOT_BASE, 0x3c000, large_block);
}
#endif
